@@ -44,6 +44,9 @@
 #include "util-profiling.h"
 #include "host.h"
 
+static int DecodeIEEE8021ah(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p,
+        uint8_t *pkt, uint16_t len, PacketQueue *pq);
+
 /**
  * \internal
  * \brief this function is used to decode IEEE802.1q packets
@@ -117,6 +120,10 @@ int DecodeVLAN(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, u
                         len - VLAN_HEADER_LEN, pq);
             }
             break;
+        case ETHERNET_TYPE_8021AH:
+            DecodeIEEE8021ah(tv, dtv, p, pkt + VLAN_HEADER_LEN,
+                    len - VLAN_HEADER_LEN, pq);
+            break;
         default:
             SCLogDebug("unknown VLAN type: %" PRIx32 "", proto);
             ENGINE_SET_INVALID_EVENT(p, VLAN_UNKNOWN_TYPE);
@@ -139,6 +146,38 @@ uint16_t DecodeVLANGetId(const Packet *p, uint8_t layer)
     return 0;
 }
 
+typedef struct IEEE8021ahHdr_ {
+    uint32_t flags;
+    uint8_t c_destination[6];
+    uint8_t c_source[6];
+    uint16_t type;              /**< next protocol */
+}  __attribute__((__packed__)) IEEE8021ahHdr;
+
+#define IEEE8021AH_HEADER_LEN sizeof(IEEE8021ahHdr)
+
+static int DecodeIEEE8021ah(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, uint16_t len, PacketQueue *pq)
+{
+    StatsIncr(tv, dtv->counter_ieee8021ah);
+
+    if (len < IEEE8021AH_HEADER_LEN) {
+        ENGINE_SET_INVALID_EVENT(p, IEEE8021AH_HEADER_TOO_SMALL);
+        return TM_ECODE_FAILED;
+    }
+
+    IEEE8021ahHdr *hdr = (IEEE8021ahHdr *)pkt;
+    uint16_t next_proto = ntohs(hdr->type);
+
+    switch (next_proto) {
+        case ETHERNET_TYPE_VLAN:
+        case ETHERNET_TYPE_8021QINQ: {
+            DecodeVLAN(tv, dtv, p, pkt + IEEE8021AH_HEADER_LEN,
+                    len - IEEE8021AH_HEADER_LEN, pq);
+            break;
+        }
+    }
+    return TM_ECODE_OK;
+}
+
 #ifdef UNITTESTS
 /** \todo Must GRE+VLAN and Multi-Vlan packets to
  * create more tests
