@@ -46,8 +46,9 @@
 static pcre *parse_regex;
 static pcre_extra *parse_regex_study;
 
-int DetectICodeMatch(ThreadVars *, DetectEngineThreadCtx *, Packet *, Signature *, const SigMatchCtx *);
-static int DetectICodeSetup(DetectEngineCtx *, Signature *, char *);
+static int DetectICodeMatch(ThreadVars *, DetectEngineThreadCtx *, Packet *,
+        const Signature *, const SigMatchCtx *);
+static int DetectICodeSetup(DetectEngineCtx *, Signature *, const char *);
 void DetectICodeRegisterTests(void);
 void DetectICodeFree(void *);
 
@@ -115,7 +116,8 @@ static inline int ICodeMatch(const uint8_t pcode, const uint8_t mode,
  * \retval 0 no match
  * \retval 1 match
  */
-int DetectICodeMatch (ThreadVars *t, DetectEngineThreadCtx *det_ctx, Packet *p, Signature *s, const SigMatchCtx *ctx)
+static int DetectICodeMatch (ThreadVars *t, DetectEngineThreadCtx *det_ctx, Packet *p,
+        const Signature *s, const SigMatchCtx *ctx)
 {
     if (PKT_IS_PSEUDOPKT(p))
         return 0;
@@ -142,7 +144,7 @@ int DetectICodeMatch (ThreadVars *t, DetectEngineThreadCtx *det_ctx, Packet *p,
  * \retval icd pointer to DetectICodeData on success
  * \retval NULL on failure
  */
-DetectICodeData *DetectICodeParse(char *icodestr)
+static DetectICodeData *DetectICodeParse(const char *icodestr)
 {
     DetectICodeData *icd = NULL;
     char *args[3] = {NULL, NULL, NULL};
@@ -246,7 +248,7 @@ error:
  * \retval 0 on Success
  * \retval -1 on Failure
  */
-static int DetectICodeSetup(DetectEngineCtx *de_ctx, Signature *s, char *icodestr)
+static int DetectICodeSetup(DetectEngineCtx *de_ctx, Signature *s, const char *icodestr)
 {
 
     DetectICodeData *icd = NULL;
@@ -340,7 +342,7 @@ static int PrefilterSetupICode(SigGroupHead *sgh)
 static _Bool PrefilterICodeIsPrefilterable(const Signature *s)
 {
     const SigMatch *sm;
-    for (sm = s->sm_lists[DETECT_SM_LIST_MATCH] ; sm != NULL; sm = sm->next) {
+    for (sm = s->init_data->smlists[DETECT_SM_LIST_MATCH] ; sm != NULL; sm = sm->next) {
         switch (sm->type) {
             case DETECT_ICODE:
                 return TRUE;
@@ -356,7 +358,7 @@ static _Bool PrefilterICodeIsPrefilterable(const Signature *s)
 /**
  * \test DetectICodeParseTest01 is a test for setting a valid icode value
  */
-int DetectICodeParseTest01(void)
+static int DetectICodeParseTest01(void)
 {
     DetectICodeData *icd = NULL;
     int result = 0;
@@ -373,7 +375,7 @@ int DetectICodeParseTest01(void)
  * \test DetectICodeParseTest02 is a test for setting a valid icode value
  *       with ">" operator
  */
-int DetectICodeParseTest02(void)
+static int DetectICodeParseTest02(void)
 {
     DetectICodeData *icd = NULL;
     int result = 0;
@@ -390,7 +392,7 @@ int DetectICodeParseTest02(void)
  * \test DetectICodeParseTest03 is a test for setting a valid icode value
  *       with "<" operator
  */
-int DetectICodeParseTest03(void)
+static int DetectICodeParseTest03(void)
 {
     DetectICodeData *icd = NULL;
     int result = 0;
@@ -407,7 +409,7 @@ int DetectICodeParseTest03(void)
  * \test DetectICodeParseTest04 is a test for setting a valid icode value
  *       with "<>" operator
  */
-int DetectICodeParseTest04(void)
+static int DetectICodeParseTest04(void)
 {
     DetectICodeData *icd = NULL;
     int result = 0;
@@ -424,7 +426,7 @@ int DetectICodeParseTest04(void)
  * \test DetectICodeParseTest05 is a test for setting a valid icode value
  *       with spaces all around
  */
-int DetectICodeParseTest05(void)
+static int DetectICodeParseTest05(void)
 {
     DetectICodeData *icd = NULL;
     int result = 0;
@@ -441,7 +443,7 @@ int DetectICodeParseTest05(void)
  * \test DetectICodeParseTest06 is a test for setting a valid icode value
  *       with ">" operator and spaces all around
  */
-int DetectICodeParseTest06(void)
+static int DetectICodeParseTest06(void)
 {
     DetectICodeData *icd = NULL;
     int result = 0;
@@ -458,7 +460,7 @@ int DetectICodeParseTest06(void)
  * \test DetectICodeParseTest07 is a test for setting a valid icode value
  *       with "<>" operator and spaces all around
  */
-int DetectICodeParseTest07(void)
+static int DetectICodeParseTest07(void)
 {
     DetectICodeData *icd = NULL;
     int result = 0;
@@ -474,7 +476,7 @@ int DetectICodeParseTest07(void)
 /**
  * \test DetectICodeParseTest08 is a test for setting an invalid icode value
  */
-int DetectICodeParseTest08(void)
+static int DetectICodeParseTest08(void)
 {
     DetectICodeData *icd = NULL;
     icd = DetectICodeParse("> 8 <> 20");
@@ -489,7 +491,7 @@ int DetectICodeParseTest08(void)
  *       keyword by creating 5 rules and matching a crafted packet against
  *       them. 4 out of 5 rules shall trigger.
  */
-int DetectICodeMatchTest01(void)
+static int DetectICodeMatchTest01(void)
 {
 
     Packet *p = NULL;
