@@ -53,12 +53,15 @@
 
 #include "output.h"
 #include "output-json.h"
+#include "output-json-alert.h"
 #include "output-json-dnp3.h"
 #include "output-json-http.h"
 #include "output-json-tls.h"
 #include "output-json-ssh.h"
 #include "output-json-smtp.h"
 #include "output-json-email-common.h"
+#include "output-json-nfs.h"
+#include "output-json-flow.h"
 
 #include "util-byte.h"
 #include "util-privs.h"
@@ -67,20 +70,28 @@
 #include "util-optimize.h"
 #include "util-buffer.h"
 #include "util-crypt.h"
+#include "util-validate.h"
 
 #define MODULE_NAME "JsonAlertLog"
 
 #ifdef HAVE_LIBJANSSON
 
-#define LOG_JSON_PAYLOAD        0x001
-#define LOG_JSON_PACKET         0x002
-#define LOG_JSON_PAYLOAD_BASE64 0x004
-#define LOG_JSON_HTTP           0x008
-#define LOG_JSON_TLS            0x010
-#define LOG_JSON_SSH            0x020
-#define LOG_JSON_SMTP           0x040
-#define LOG_JSON_TAGGED_PACKETS 0x080
-#define LOG_JSON_DNP3           0x100
+#define LOG_JSON_PAYLOAD           BIT_U16(0)
+#define LOG_JSON_PACKET            BIT_U16(1)
+#define LOG_JSON_PAYLOAD_BASE64    BIT_U16(2)
+#define LOG_JSON_HTTP              BIT_U16(3)
+#define LOG_JSON_TLS               BIT_U16(4)
+#define LOG_JSON_SSH               BIT_U16(5)
+#define LOG_JSON_SMTP              BIT_U16(6)
+#define LOG_JSON_TAGGED_PACKETS    BIT_U16(7)
+#define LOG_JSON_DNP3              BIT_U16(8)
+#define LOG_JSON_VARS              BIT_U16(9)
+#define LOG_JSON_APP_LAYER         BIT_U16(10)
+#define LOG_JSON_FLOW              BIT_U16(11)
+#define LOG_JSON_HTTP_BODY         BIT_U16(12)
+#define LOG_JSON_HTTP_BODY_BASE64  BIT_U16(13)
+
+#define LOG_JSON_METADATA_ALL  (LOG_JSON_APP_LAYER|LOG_JSON_HTTP|LOG_JSON_TLS|LOG_JSON_SSH|LOG_JSON_SMTP|LOG_JSON_DNP3|LOG_JSON_VARS|LOG_JSON_FLOW)
 
 #define JSON_STREAM_BUFFER_SIZE 4096
 
@@ -101,7 +112,7 @@ typedef struct JsonAlertLogThread_ {
 
 /* Callback function to pack payload contents from a stream into a buffer
  * so we can report them in JSON output. */
-static int AlertJsonDumpStreamSegmentCallback(const Packet *p, void *data, uint8_t *buf, uint32_t buflen)
+static int AlertJsonDumpStreamSegmentCallback(const Packet *p, void *data, const uint8_t *buf, uint32_t buflen)
 {
     MemBuffer *payload = (MemBuffer *)data;
     MemBufferWriteRaw(payload, buf, buflen);
@@ -172,9 +183,57 @@ static void AlertJsonDnp3(const Flow *f, json_t *js)
     return;
 }
 
+static void AlertJsonSourceTarget(const Packet *p, const PacketAlert *pa,
+                                  json_t *js, json_t* ajs)
+{
+    json_t *sjs = json_object();
+    if (sjs == NULL) {
+        return;
+    }
+
+    json_t *tjs = json_object();
+    if (tjs == NULL) {
+        json_decref(sjs);
+        return;
+    }
+
+    if (pa->s->flags & SIG_FLAG_DEST_IS_TARGET) {
+        json_object_set(sjs, "ip", json_object_get(js, "src_ip"));
+        json_object_set(tjs, "ip", json_object_get(js, "dest_ip"));
+        switch (p->proto) {
+            case IPPROTO_ICMP:
+            case IPPROTO_ICMPV6:
+                break;
+            case IPPROTO_UDP:
+            case IPPROTO_TCP:
+            case IPPROTO_SCTP:
+                json_object_set(sjs, "port", json_object_get(js, "src_port"));
+                json_object_set(tjs, "port", json_object_get(js, "dest_port"));
+                break;
+        }
+    } else if (pa->s->flags & SIG_FLAG_SRC_IS_TARGET) {
+        json_object_set(sjs, "ip", json_object_get(js, "dest_ip"));
+        json_object_set(tjs, "ip", json_object_get(js, "src_ip"));
+        switch (p->proto) {
+            case IPPROTO_ICMP:
+            case IPPROTO_ICMPV6:
+                break;
+            case IPPROTO_UDP:
+            case IPPROTO_TCP:
+            case IPPROTO_SCTP:
+                json_object_set(sjs, "port", json_object_get(js, "dest_port"));
+                json_object_set(tjs, "port", json_object_get(js, "src_port"));
+                break;
+        }
+    }
+    json_object_set_new(ajs, "source", sjs);
+    json_object_set_new(ajs, "target", tjs);
+}
+
+
 void AlertJsonHeader(const Packet *p, const PacketAlert *pa, json_t *js)
 {
-    char *action = "allowed";
+    const char *action = "allowed";
     /* use packet action if rate_filter modified the action */
     if (unlikely(pa->flags & PACKET_ALERT_RATE_FILTER_MODIFIED)) {
         if (PACKET_TEST_ACTION(p, (ACTION_DROP|ACTION_REJECT|
@@ -196,7 +255,6 @@ void AlertJsonHeader(const Packet *p, const PacketAlert *pa, json_t *js)
 
     json_t *ajs = json_object();
     if (ajs == NULL) {
-        json_decref(js);
         return;
     }
 
@@ -213,10 +271,37 @@ void AlertJsonHeader(const Packet *p, const PacketAlert *pa, json_t *js)
     if (p->tenant_id > 0)
         json_object_set_new(ajs, "tenant_id", json_integer(p->tenant_id));
 
+    if (pa->s->flags & SIG_FLAG_HAS_TARGET) {
+        AlertJsonSourceTarget(p, pa, js, ajs);
+    }
+
     /* alert */
     json_object_set_new(js, "alert", ajs);
 }
 
+static void AlertJsonTunnel(const Packet *p, json_t *js)
+{
+    json_t *tunnel = json_object();
+    if (tunnel == NULL)
+        return;
+
+    if (p->root == NULL) {
+        json_decref(tunnel);
+        return;
+    }
+
+    /* get a lock to access root packet fields */
+    SCMutex *m = &p->root->tunnel_mutex;
+
+    SCMutexLock(m);
+    JsonFiveTuple((const Packet *)p->root, 0, tunnel);
+    SCMutexUnlock(m);
+
+    json_object_set_new(tunnel, "depth", json_integer(p->recursion_level));
+
+    json_object_set_new(js, "tunnel", tunnel);
+}
+
 static void AlertJsonPacket(const Packet *p, json_t *js)
 {
     unsigned long len = GET_PKT_LEN(p) * 2;
@@ -234,6 +319,27 @@ static void AlertJsonPacket(const Packet *p, json_t *js)
     json_object_set_new(js, "packet_info", packetinfo_js);
 }
 
+static void AlertAddPayload(AlertJsonOutputCtx *json_output_ctx, json_t *js, const Packet *p)
+{
+    if (json_output_ctx->flags & LOG_JSON_PAYLOAD_BASE64) {
+        unsigned long len = p->payload_len * 2 + 1;
+        uint8_t encoded[len];
+        if (Base64Encode(p->payload, p->payload_len, encoded, &len) == SC_BASE64_OK) {
+            json_object_set_new(js, "payload", json_string((char *)encoded));
+        }
+    }
+
+    if (json_output_ctx->flags & LOG_JSON_PAYLOAD) {
+        uint8_t printable_buf[p->payload_len + 1];
+        uint32_t offset = 0;
+        PrintStringsToBuffer(printable_buf, &offset,
+                p->payload_len + 1,
+                p->payload, p->payload_len);
+        printable_buf[p->payload_len] = '\0';
+        json_object_set_new(js, "payload_printable", json_string((char *)printable_buf));
+    }
+}
+
 static int AlertJson(ThreadVars *tv, JsonAlertLogThread *aft, const Packet *p)
 {
     MemBuffer *payload = aft->payload_buffer;
@@ -260,6 +366,10 @@ static int AlertJson(ThreadVars *tv, JsonAlertLogThread *aft, const Packet *p)
         /* alert */
         AlertJsonHeader(p, pa, js);
 
+        if (IS_TUNNEL_PKT(p)) {
+            AlertJsonTunnel(p, js);
+        }
+
         if (json_output_ctx->flags & LOG_JSON_HTTP) {
             if (p->flow != NULL) {
                 uint16_t proto = FlowGetAppProtocol(p->flow);
@@ -267,8 +377,15 @@ static int AlertJson(ThreadVars *tv, JsonAlertLogThread *aft, const Packet *p)
                 /* http alert */
                 if (proto == ALPROTO_HTTP) {
                     hjs = JsonHttpAddMetadata(p->flow, pa->tx_id);
-                    if (hjs)
+                    if (hjs) {
+                        if (json_output_ctx->flags & LOG_JSON_HTTP_BODY) {
+                            JsonHttpLogJSONBodyPrintable(hjs, p->flow, pa->tx_id);
+                        }
+                        if (json_output_ctx->flags & LOG_JSON_HTTP_BODY_BASE64) {
+                            JsonHttpLogJSONBodyBase64(hjs, p->flow, pa->tx_id);
+                        }
                         json_object_set_new(js, "http", hjs);
+                    }
                 }
             }
         }
@@ -277,7 +394,7 @@ static int AlertJson(ThreadVars *tv, JsonAlertLogThread *aft, const Packet *p)
             if (p->flow != NULL) {
                 uint16_t proto = FlowGetAppProtocol(p->flow);
 
-                /* http alert */
+                /* tls alert */
                 if (proto == ALPROTO_TLS)
                     AlertJsonTls(p->flow, js);
             }
@@ -287,7 +404,7 @@ static int AlertJson(ThreadVars *tv, JsonAlertLogThread *aft, const Packet *p)
             if (p->flow != NULL) {
                 uint16_t proto = FlowGetAppProtocol(p->flow);
 
-                /* http alert */
+                /* ssh alert */
                 if (proto == ALPROTO_SSH)
                     AlertJsonSsh(p->flow, js);
             }
@@ -297,7 +414,7 @@ static int AlertJson(ThreadVars *tv, JsonAlertLogThread *aft, const Packet *p)
             if (p->flow != NULL) {
                 uint16_t proto = FlowGetAppProtocol(p->flow);
 
-                /* http alert */
+                /* smtp alert */
                 if (proto == ALPROTO_SMTP) {
                     hjs = JsonSMTPAddMetadata(p->flow, pa->tx_id);
                     if (hjs)
@@ -309,16 +426,48 @@ static int AlertJson(ThreadVars *tv, JsonAlertLogThread *aft, const Packet *p)
                 }
             }
         }
-
+#ifdef HAVE_RUST
+        if ((json_output_ctx->flags & LOG_JSON_APP_LAYER) && p->flow != NULL) {
+            uint16_t alproto = FlowGetAppProtocol(p->flow);
+            if (alproto == ALPROTO_NFS) {
+                hjs = JsonNFSAddMetadataRPC(p->flow, pa->tx_id);
+                if (hjs)
+                    json_object_set_new(js, "rpc", hjs);
+                hjs = JsonNFSAddMetadata(p->flow, pa->tx_id);
+                if (hjs)
+                    json_object_set_new(js, "nfs", hjs);
+            }
+        }
+#endif
         if (json_output_ctx->flags & LOG_JSON_DNP3) {
             if (p->flow != NULL) {
                 uint16_t proto = FlowGetAppProtocol(p->flow);
+
+                /* dnp3 alert */
                 if (proto == ALPROTO_DNP3) {
                     AlertJsonDnp3(p->flow, js);
                 }
             }
         }
 
+        if (json_output_ctx->flags & LOG_JSON_VARS) {
+            JsonAddVars(p, p->flow, js);
+        }
+
+        if (p->flow) {
+            if (json_output_ctx->flags & LOG_JSON_FLOW) {
+                hjs = json_object();
+                if (hjs != NULL) {
+                    JsonAddFlow(p->flow, js, hjs);
+                    json_object_set_new(js, "flow", hjs);
+                }
+            } else {
+                json_object_set_new(js, "app_proto",
+                        json_string(AppProtoToString(p->flow->alproto)));
+            }
+        }
+
+
         /* payload */
         if (json_output_ctx->flags & (LOG_JSON_PAYLOAD | LOG_JSON_PAYLOAD_BASE64)) {
             int stream = (p->proto == IPPROTO_TCP) ?
@@ -340,42 +489,30 @@ static int AlertJson(ThreadVars *tv, JsonAlertLogThread *aft, const Packet *p)
                 StreamSegmentForEach((const Packet *)p, flag,
                                     AlertJsonDumpStreamSegmentCallback,
                                     (void *)payload);
+                if (payload->offset) {
+                    if (json_output_ctx->flags & LOG_JSON_PAYLOAD_BASE64) {
+                        unsigned long len = json_output_ctx->payload_buffer_size * 2;
+                        uint8_t encoded[len];
+                        Base64Encode(payload->buffer, payload->offset, encoded, &len);
+                        json_object_set_new(js, "payload", json_string((char *)encoded));
+                    }
 
-                if (json_output_ctx->flags & LOG_JSON_PAYLOAD_BASE64) {
-                    unsigned long len = json_output_ctx->payload_buffer_size * 2;
-                    uint8_t encoded[len];
-                    Base64Encode(payload->buffer, payload->offset, encoded, &len);
-                    json_object_set_new(js, "payload", json_string((char *)encoded));
-                }
-
-                if (json_output_ctx->flags & LOG_JSON_PAYLOAD) {
-                    uint8_t printable_buf[payload->offset + 1];
-                    uint32_t offset = 0;
-                    PrintStringsToBuffer(printable_buf, &offset,
-                                     sizeof(printable_buf),
-                                     payload->buffer, payload->offset);
-                    json_object_set_new(js, "payload_printable",
-                                        json_string((char *)printable_buf));
+                    if (json_output_ctx->flags & LOG_JSON_PAYLOAD) {
+                        uint8_t printable_buf[payload->offset + 1];
+                        uint32_t offset = 0;
+                        PrintStringsToBuffer(printable_buf, &offset,
+                                sizeof(printable_buf),
+                                payload->buffer, payload->offset);
+                        json_object_set_new(js, "payload_printable",
+                                json_string((char *)printable_buf));
+                    }
+                } else if (p->payload_len) {
+                    /* Fallback on packet payload */
+                    AlertAddPayload(json_output_ctx, js, p);
                 }
             } else {
                 /* This is a single packet and not a stream */
-                if (json_output_ctx->flags & LOG_JSON_PAYLOAD_BASE64) {
-                    unsigned long len = p->payload_len * 2 + 1;
-                    uint8_t encoded[len];
-                    if (Base64Encode(p->payload, p->payload_len, encoded, &len) == SC_BASE64_OK) {
-                        json_object_set_new(js, "payload", json_string((char *)encoded));
-                    }
-                }
-
-                if (json_output_ctx->flags & LOG_JSON_PAYLOAD) {
-                    uint8_t printable_buf[p->payload_len + 1];
-                    uint32_t offset = 0;
-                    PrintStringsToBuffer(printable_buf, &offset,
-                                     p->payload_len + 1,
-                                     p->payload, p->payload_len);
-                    printable_buf[p->payload_len] = '\0';
-                    json_object_set_new(js, "payload_printable", json_string((char *)printable_buf));
-                }
+                AlertAddPayload(json_output_ctx, js, p);
             }
 
             json_object_set_new(js, "stream", json_integer(stream));
@@ -454,7 +591,7 @@ static int AlertJsonDecoderEvent(ThreadVars *tv, JsonAlertLogThread *aft, const
             continue;
         }
 
-        char *action = "allowed";
+        const char *action = "allowed";
         if (pa->action & (ACTION_REJECT|ACTION_REJECT_DST|ACTION_REJECT_BOTH)) {
             action = "blocked";
         } else if ((pa->action & ACTION_DROP) && EngineModeIsIPS()) {
@@ -528,7 +665,7 @@ static int JsonAlertLogCondition(ThreadVars *tv, const Packet *p)
 }
 
 #define OUTPUT_BUFFER_SIZE 65535
-static TmEcode JsonAlertLogThreadInit(ThreadVars *t, void *initdata, void **data)
+static TmEcode JsonAlertLogThreadInit(ThreadVars *t, const void *initdata, void **data)
 {
     JsonAlertLogThread *aft = SCMalloc(sizeof(JsonAlertLogThread));
     if (unlikely(aft == NULL))
@@ -554,10 +691,11 @@ static TmEcode JsonAlertLogThreadInit(ThreadVars *t, void *initdata, void **data
 
     aft->payload_buffer = MemBufferCreateNew(json_output_ctx->payload_buffer_size);
     if (aft->payload_buffer == NULL) {
+        MemBufferFree(aft->json_buffer);
         SCFree(aft);
         return TM_ECODE_FAILED;
     }
-    
+
     *data = (void *)aft;
     return TM_ECODE_OK;
 }
@@ -610,6 +748,19 @@ static void JsonAlertLogDeInitCtxSub(OutputCtx *output_ctx)
     SCFree(output_ctx);
 }
 
+static void SetFlag(const ConfNode *conf, const char *name, uint16_t flag, uint16_t *out_flags)
+{
+    DEBUG_VALIDATE_BUG_ON(conf == NULL);
+    const char *setting = ConfNodeLookupChildValue(conf, name);
+    if (setting != NULL) {
+        if (ConfValIsTrue(setting)) {
+            *out_flags |= flag;
+        } else {
+            *out_flags &= ~flag;
+        }
+    }
+}
+
 #define DEFAULT_LOG_FILENAME "alert.json"
 
 static void XffSetup(AlertJsonOutputCtx *json_output_ctx, ConfNode *conf)
@@ -627,47 +778,25 @@ static void XffSetup(AlertJsonOutputCtx *json_output_ctx, ConfNode *conf)
     uint32_t payload_buffer_size = JSON_STREAM_BUFFER_SIZE;
 
     if (conf != NULL) {
-        const char *payload = ConfNodeLookupChildValue(conf, "payload");
+        SetFlag(conf, "metadata", LOG_JSON_METADATA_ALL, &json_output_ctx->flags);
+        SetFlag(conf, "flow", LOG_JSON_FLOW, &json_output_ctx->flags);
+        SetFlag(conf, "vars", LOG_JSON_VARS, &json_output_ctx->flags);
+
+        SetFlag(conf, "http", LOG_JSON_HTTP, &json_output_ctx->flags);
+        SetFlag(conf, "tls",  LOG_JSON_TLS,  &json_output_ctx->flags);
+        SetFlag(conf, "ssh",  LOG_JSON_SSH,  &json_output_ctx->flags);
+        SetFlag(conf, "smtp", LOG_JSON_SMTP, &json_output_ctx->flags);
+        SetFlag(conf, "dnp3", LOG_JSON_DNP3, &json_output_ctx->flags);
+
+        SetFlag(conf, "payload", LOG_JSON_PAYLOAD_BASE64, &json_output_ctx->flags);
+        SetFlag(conf, "packet", LOG_JSON_PACKET, &json_output_ctx->flags);
+        SetFlag(conf, "tagged-packets", LOG_JSON_TAGGED_PACKETS, &json_output_ctx->flags);
+        SetFlag(conf, "payload-printable", LOG_JSON_PAYLOAD, &json_output_ctx->flags);
+        SetFlag(conf, "http-body-printable", LOG_JSON_HTTP_BODY, &json_output_ctx->flags);
+        SetFlag(conf, "http-body", LOG_JSON_HTTP_BODY_BASE64, &json_output_ctx->flags);
+
         const char *payload_buffer_value = ConfNodeLookupChildValue(conf, "payload-buffer-size");
-        const char *packet  = ConfNodeLookupChildValue(conf, "packet");
-        const char *payload_printable = ConfNodeLookupChildValue(conf, "payload-printable");
-        const char *http = ConfNodeLookupChildValue(conf, "http");
-        const char *tls = ConfNodeLookupChildValue(conf, "tls");
-        const char *ssh = ConfNodeLookupChildValue(conf, "ssh");
-        const char *smtp = ConfNodeLookupChildValue(conf, "smtp");
-        const char *tagged_packets = ConfNodeLookupChildValue(conf, "tagged-packets");
-        const char *dnp3 = ConfNodeLookupChildValue(conf, "dnp3");
 
-        if (ssh != NULL) {
-            if (ConfValIsTrue(ssh)) {
-                json_output_ctx->flags |= LOG_JSON_SSH;
-            }
-        }
-        if (tls != NULL) {
-            if (ConfValIsTrue(tls)) {
-                json_output_ctx->flags |= LOG_JSON_TLS;
-            }
-        }
-        if (http != NULL) {
-            if (ConfValIsTrue(http)) {
-                json_output_ctx->flags |= LOG_JSON_HTTP;
-            }
-        }
-        if (smtp != NULL) {
-            if (ConfValIsTrue(smtp)) {
-                json_output_ctx->flags |= LOG_JSON_SMTP;
-            }
-        }
-        if (payload_printable != NULL) {
-            if (ConfValIsTrue(payload_printable)) {
-                json_output_ctx->flags |= LOG_JSON_PAYLOAD;
-            }
-        }
-        if (payload != NULL) {
-            if (ConfValIsTrue(payload)) {
-                json_output_ctx->flags |= LOG_JSON_PAYLOAD_BASE64;
-            }
-        }
         if (payload_buffer_value != NULL) {
             uint32_t value;
             if (ParseSizeStringU32(payload_buffer_value, &value) < 0) {
@@ -679,23 +808,8 @@ static void XffSetup(AlertJsonOutputCtx *json_output_ctx, ConfNode *conf)
                 payload_buffer_size = value;
             }
         }
-        if (packet != NULL) {
-            if (ConfValIsTrue(packet)) {
-                json_output_ctx->flags |= LOG_JSON_PACKET;
-            }
-        }
-        if (tagged_packets != NULL) {
-            if (ConfValIsTrue(tagged_packets)) {
-                json_output_ctx->flags |= LOG_JSON_TAGGED_PACKETS;
-            }
-        }
-        if (dnp3 != NULL) {
-            if (ConfValIsTrue(dnp3)) {
-                json_output_ctx->flags |= LOG_JSON_DNP3;
-            }
-        }
 
-	json_output_ctx->payload_buffer_size = payload_buffer_size;
+        json_output_ctx->payload_buffer_size = payload_buffer_size;
         HttpXFFGetCfg(conf, xff_cfg);
     }
 }
