@@ -68,6 +68,10 @@ static json_t *OutputStats2Json(json_t *js, const char *key)
     const char *dot = index(key, '.');
     if (dot == NULL)
         return NULL;
+    if (strlen(dot) > 2) {
+        if (*(dot + 1) == '.' && *(dot + 2) != '\0')
+            dot = index(dot + 2, '.');
+    }
 
     size_t predot_len = (dot - key) + 1;
     char s[predot_len];
@@ -102,8 +106,9 @@ json_t *StatsToJSON(const StatsTable *st, uint8_t flags)
     }
 
     /* Uptime, in seconds. */
+    double up_time_d = difftime(tval.tv_sec, st->start_time);
     json_object_set_new(js_stats, "uptime",
-        json_integer((int)difftime(tval.tv_sec, st->start_time)));
+        json_integer((int)up_time_d));
 
     uint32_t u = 0;
     if (flags & JSON_STATS_TOTALS) {
@@ -207,7 +212,7 @@ static int JsonStatsLogger(ThreadVars *tv, void *thread_data, const StatsTable *
 }
 
 #define OUTPUT_BUFFER_SIZE 65535
-static TmEcode JsonStatsLogThreadInit(ThreadVars *t, void *initdata, void **data)
+static TmEcode JsonStatsLogThreadInit(ThreadVars *t, const void *initdata, void **data)
 {
     JsonStatsLogThread *aft = SCMalloc(sizeof(JsonStatsLogThread));
     if (unlikely(aft == NULL))
@@ -261,7 +266,7 @@ static void OutputStatsLogDeinit(OutputCtx *output_ctx)
 }
 
 #define DEFAULT_LOG_FILENAME "stats.json"
-OutputCtx *OutputStatsLogInit(ConfNode *conf)
+static OutputCtx *OutputStatsLogInit(ConfNode *conf)
 {
     LogFileCtx *file_ctx = LogFileNewCtx();
     if(file_ctx == NULL) {
@@ -321,7 +326,7 @@ static void OutputStatsLogDeinitSub(OutputCtx *output_ctx)
     SCFree(output_ctx);
 }
 
-OutputCtx *OutputStatsLogInitSub(ConfNode *conf, OutputCtx *parent_ctx)
+static OutputCtx *OutputStatsLogInitSub(ConfNode *conf, OutputCtx *parent_ctx)
 {
     AlertJsonThread *ajt = parent_ctx->data;
 
