@@ -325,6 +325,13 @@ typedef enum {
     SC_WARN_REMOVE_FILE,
     SC_ERR_NO_MAGIC_SUPPORT,
     SC_ERR_REDIS,
+    SC_ERR_VAR_LIMIT,
+    SC_WARN_DUPLICATE_OUTPUT,
+    SC_WARN_CHMOD,
+    SC_WARN_LOG_CF_TOO_MANY_NODES,
+    SC_WARN_EVENT_DROPPED,
+    SC_ERR_NO_REDIS_ASYNC,
+    SC_ERR_REDIS_CONFIG
 } SCError;
 
 const char *SCErrorToString(SCError);
