@@ -149,7 +149,6 @@ void FileForceTrackingEnable(void)
     g_file_force_tracking = 1;
 }
 
-
 /**
  * \brief Function to parse forced file hashing configuration.
  */
@@ -260,7 +259,7 @@ uint16_t FileFlowToFlags(const Flow *flow, uint8_t direction)
     return flags;
 }
 
-int FileMagicSize(void)
+static int FileMagicSize(void)
 {
     /** \todo make this size configurable */
     return 512;
@@ -343,21 +342,26 @@ static int FilePruneFile(File *file)
 void FilePrune(FileContainer *ffc)
 {
     File *file = ffc->head;
+    File *prev = NULL;
 
     while (file) {
-        if (FilePruneFile(file) == 0)
-            break;
-
-        BUG_ON(file != ffc->head);
+        if (FilePruneFile(file) == 0) {
+            prev = file;
+            file = file->next;
+            continue;
+        }
 
         SCLogDebug("removing file %p", file);
 
         File *file_next = file->next;
 
+        if (prev)
+            prev->next = file_next;
         /* update head and tail */
-        ffc->head = file_next;
+        if (file == ffc->head)
+            ffc->head = file_next;
         if (file == ffc->tail)
-            ffc->tail = NULL;
+            ffc->tail = prev;
 
         FileFree(file);
         file = file_next;
@@ -512,6 +516,13 @@ int FileSetTx(File *ff, uint64_t txid)
     SCReturnInt(0);
 }
 
+void FileContainerSetTx(FileContainer *ffc, uint64_t tx_id)
+{
+    if (ffc && ffc->tail) {
+        (void)FileSetTx(ffc->tail, tx_id);
+    }
+}
+
 /**
  *  \brief check if we have stored enough
  *
@@ -559,11 +570,76 @@ static int AppendData(File *file, const uint8_t *data, uint32_t data_len)
     SCReturnInt(0);
 }
 
-/**
- *  \brief Store a chunk of file data in the flow. The open "flowfile"
- *         will be used.
+/** \internal
+ *  \brief Store/handle a chunk of file data in the File structure
  *
- *  \param ffc the container
+ *  \param ff the file
+ *  \param data data chunk
+ *  \param data_len data chunk len
+ *
+ *  \retval  0 ok
+ *  \retval -1 error
+ *  \retval -2 no store for this file
+ */
+static int FileAppendDataDo(File *ff, const uint8_t *data, uint32_t data_len)
+{
+    SCEnter();
+#ifdef DEBUG_VALIDATION
+    BUG_ON(ff == NULL);
+#endif
+
+    ff->size += data_len;
+
+    if (ff->state != FILE_STATE_OPENED) {
+        if (ff->flags & FILE_NOSTORE) {
+            SCReturnInt(-2);
+        }
+        SCReturnInt(-1);
+    }
+
+    if (FileStoreNoStoreCheck(ff) == 1) {
+#ifdef HAVE_NSS
+        int hash_done = 0;
+        /* no storage but forced hashing */
+        if (ff->md5_ctx) {
+            HASH_Update(ff->md5_ctx, data, data_len);
+            hash_done = 1;
+        }
+        if (ff->sha1_ctx) {
+            HASH_Update(ff->sha1_ctx, data, data_len);
+            hash_done = 1;
+        }
+        if (ff->sha256_ctx) {
+            HASH_Update(ff->sha256_ctx, data, data_len);
+            hash_done = 1;
+        }
+
+        if (hash_done)
+            SCReturnInt(0);
+#endif
+        if (g_file_force_tracking || (!(ff->flags & FILE_NOTRACK)))
+            SCReturnInt(0);
+
+        ff->state = FILE_STATE_TRUNCATED;
+        SCLogDebug("flowfile state transitioned to FILE_STATE_TRUNCATED");
+        SCReturnInt(-2);
+    }
+
+    SCLogDebug("appending %"PRIu32" bytes", data_len);
+
+    if (AppendData(ff, data, data_len) != 0) {
+        ff->state = FILE_STATE_ERROR;
+        SCReturnInt(-1);
+    }
+
+    SCReturnInt(0);
+}
+
+/**
+ *  \brief Store/handle a chunk of file data in the File structure
+ *         The last file in the FileContainer will be used.
+ *
+ *  \param ffc FileContainer used to append to
  *  \param data data chunk
  *  \param data_len data chunk len
  *
@@ -578,52 +654,75 @@ int FileAppendData(FileContainer *ffc, const uint8_t *data, uint32_t data_len)
     if (ffc == NULL || ffc->tail == NULL || data == NULL || data_len == 0) {
         SCReturnInt(-1);
     }
+    int r = FileAppendDataDo(ffc->tail, data, data_len);
+    SCReturnInt(r);
+}
 
-    ffc->tail->size += data_len;
+/**
+ *  \brief Store/handle a chunk of file data in the File structure
+ *         The file with 'track_id' in the FileContainer will be used.
+ *
+ *  \param ffc FileContainer used to append to
+ *  \param track_id id to lookup the file
+ *  \param data data chunk
+ *  \param data_len data chunk len
+ *
+ *  \retval  0 ok
+ *  \retval -1 error
+ *  \retval -2 no store for this file
+ */
+int FileAppendDataById(FileContainer *ffc, uint32_t track_id,
+        const uint8_t *data, uint32_t data_len)
+{
+    SCEnter();
 
-    if (ffc->tail->state != FILE_STATE_OPENED) {
-        if (ffc->tail->flags & FILE_NOSTORE) {
-            SCReturnInt(-2);
-        }
+    if (ffc == NULL || ffc->tail == NULL || data == NULL || data_len == 0) {
         SCReturnInt(-1);
     }
-
-    if (FileStoreNoStoreCheck(ffc->tail) == 1) {
-#ifdef HAVE_NSS
-        int hash_done = 0;
-        /* no storage but forced hashing */
-        if (ffc->tail->md5_ctx) {
-            HASH_Update(ffc->tail->md5_ctx, data, data_len);
-            hash_done = 1;
+    File *ff = ffc->head;
+    for ( ; ff != NULL; ff = ff->next) {
+        if (track_id == ff->file_track_id) {
+            int r = FileAppendDataDo(ff, data, data_len);
+            SCReturnInt(r);
         }
-        if (ffc->tail->sha1_ctx) {
-            HASH_Update(ffc->tail->sha1_ctx, data, data_len);
-            hash_done = 1;
-        }
-        if (ffc->tail->sha256_ctx) {
-            HASH_Update(ffc->tail->sha256_ctx, data, data_len);
-            hash_done = 1;
-        }
-
-        if (hash_done)
-            SCReturnInt(0);
-#endif
-        if (g_file_force_tracking || (!(ffc->tail->flags & FILE_NOTRACK)))
-            SCReturnInt(0);
-
-        ffc->tail->state = FILE_STATE_TRUNCATED;
-        SCLogDebug("flowfile state transitioned to FILE_STATE_TRUNCATED");
-        SCReturnInt(-2);
     }
+    SCReturnInt(-1);
+}
 
-    SCLogDebug("appending %"PRIu32" bytes", data_len);
+/**
+ *  \brief Store/handle a chunk of file data in the File structure
+ *         The file with 'track_id' in the FileContainer will be used.
+ *
+ *  \param ffc FileContainer used to append to
+ *  \param track_id id to lookup the file
+ *  \param data data chunk
+ *  \param data_len data chunk len
+ *
+ *  \retval  0 ok
+ *  \retval -1 error
+ *  \retval -2 no store for this file
+ */
+int FileAppendGAPById(FileContainer *ffc, uint32_t track_id,
+        const uint8_t *data, uint32_t data_len)
+{
+    SCEnter();
 
-    if (AppendData(ffc->tail, data, data_len) != 0) {
-        ffc->tail->state = FILE_STATE_ERROR;
+    if (ffc == NULL || ffc->tail == NULL || data == NULL || data_len == 0) {
         SCReturnInt(-1);
     }
+    File *ff = ffc->head;
+    for ( ; ff != NULL; ff = ff->next) {
+        if (track_id == ff->file_track_id) {
+            ff->flags |= FILE_HAS_GAPS;
+            ff->flags |= (FILE_NOMD5|FILE_NOSHA1|FILE_NOSHA256);
+            ff->flags &= ~(FILE_MD5|FILE_SHA1|FILE_SHA256);
+            SCLogDebug("FILE_HAS_GAPS set");
 
-    SCReturnInt(0);
+            int r = FileAppendDataDo(ff, data, data_len);
+            SCReturnInt(r);
+        }
+    }
+    SCReturnInt(-1);
 }
 
 /**
@@ -712,6 +811,8 @@ File *FileOpenFile(FileContainer *ffc, const StreamingBufferConfig *sbcfg,
     ff->state = FILE_STATE_OPENED;
     SCLogDebug("flowfile state transitioned to FILE_STATE_OPENED");
 
+    ff->fd = -1;
+
     FileContainerAdd(ffc, ff);
 
     if (data != NULL) {
@@ -725,6 +826,18 @@ File *FileOpenFile(FileContainer *ffc, const StreamingBufferConfig *sbcfg,
 
     SCReturnPtr(ff, "File");
 }
+File *FileOpenFileWithId(FileContainer *ffc, const StreamingBufferConfig *sbcfg,
+        uint32_t track_id, const uint8_t *name, uint16_t name_len,
+        const uint8_t *data, uint32_t data_len, uint16_t flags)
+{
+    File *ff = FileOpenFile(ffc, sbcfg, name, name_len, data, data_len, flags);
+    if (ff == NULL)
+        return NULL;
+
+    ff->file_track_id = track_id;
+    ff->flags |= FILE_USE_TRACKID;
+    return ff;
+}
 
 static int FileCloseFilePtr(File *ff, const uint8_t *data,
         uint32_t data_len, uint16_t flags)
@@ -759,7 +872,7 @@ static int FileCloseFilePtr(File *ff, const uint8_t *data,
         }
     }
 
-    if (flags & FILE_TRUNCATED) {
+    if ((flags & FILE_TRUNCATED) || (ff->flags & FILE_HAS_GAPS)) {
         ff->state = FILE_STATE_TRUNCATED;
         SCLogDebug("flowfile state transitioned to FILE_STATE_TRUNCATED");
 
@@ -819,6 +932,26 @@ int FileCloseFile(FileContainer *ffc, const uint8_t *data,
 
     SCReturnInt(0);
 }
+int FileCloseFileById(FileContainer *ffc, uint32_t track_id,
+        const uint8_t *data, uint32_t data_len, uint16_t flags)
+{
+    SCEnter();
+
+    if (ffc == NULL || ffc->tail == NULL) {
+        SCReturnInt(-1);
+    }
+
+    File *ff = ffc->head;
+    for ( ; ff != NULL; ff = ff->next) {
+        if (track_id == ff->file_track_id) {
+            if (FileCloseFilePtr(ff, data, data_len, flags) == -1) {
+                SCReturnInt(-1);
+            }
+            SCReturnInt(0);
+        }
+    }
+    SCReturnInt(-1);
+}
 
 /**
  *  \brief disable file storage for a flow
@@ -1037,7 +1170,7 @@ void FileDisableFilesize(Flow *f, uint8_t direction)
  *
  *  \param ff file
  */
-void FileDisableStoringForFile(File *ff)
+static void FileDisableStoringForFile(File *ff)
 {
     SCEnter();
 
@@ -1103,7 +1236,7 @@ void FileStoreFileById(FileContainer *fc, uint32_t file_id)
 
     if (fc != NULL) {
         for (ptr = fc->head; ptr != NULL; ptr = ptr->next) {
-            if (ptr->file_id == file_id) {
+            if (ptr->file_store_id == file_id) {
                 FileStore(ptr);
             }
         }
